==> apracjs-13.1.29.tgz: package/lib/database/base/baseLib.js <==
﻿var StringBuilder = require('../../common/stringBuilder');
var Url = require('url');
var DebugDBHelper = require('debug')('aprac:DBHelper');
var SqlString = require('sqlstring');
var StringUtils = require("../../common/stringUtils");
var Cuid = require('../../common/cuid');
var Log = require('../../log/log4Helper').getInstance();
var ApracConfig = require('../../config').apracConfig;

class BaseLib {
    constructor(connectionSetting) {
        this.connectionSetting = connectionSetting;
        this.connectionStatusEnum = new Object();
        this.connectionStatusEnum.Open = 1;
        this.connectionStatusEnum.Close = 0;
        this._initConnectionSetting();
    }

    _initConnectionSetting() {
        let uriInfo = Url.parse(this.connectionSetting, true);
        if (uriInfo.query.useConnectionPooling) {
            this.isPool = true;
            this.useConnectionPooling = uriInfo.query.useConnectionPooling;
        } else {
            this.isPool = false;
        }

        this.hostname = uriInfo.hostname;
        this.port = uriInfo.port;
        this.user = uriInfo.auth.split(':')[0];
        this.password = uriInfo.auth.split(':')[1];
        this.database = uriInfo.pathname.indexOf('/') == 0 ? uriInfo.pathname.substr(1) : uriInfo.pathname;
        this.connectionlimit = uriInfo.query && uriInfo.query.connectionLimit ? parseInt(uriInfo.query.connectionLimit) : 10;
        if (isNaN(this.connectionlimit)) {
            this.connectionlimit = 10;
        }
        this.connection = null;
        this.sqlInterceptor = null;
        this.connectionStatus = this.connectionStatusEnum.Close; //0：未连接 1：已连接
        this.hasTransaction = false;
    }

    get connectionlimit() {
        return this._connectionlimit;
    }

    set connectionlimit(value) {
        this._connectionlimit = value;
    }

    get hostname() {
        return this._hostname;
    }

    set hostname(value) {
        this._hostname = value;
    }

    get port() {
        return this._port;
    }

    set port(value) {
        this._port = value;
    }

    get user() {
        return this._user;
    }

    set user(value) {
        this._user = value;
    }

    get password() {
        return this._password;
    }

    set password(value) {
        this._password = value;
    }

    get database() {
        return this._database;
    }

    set database(value) {
        this._database = value;
    }

    get isPool() {
        return this._isPool;
    }

    set isPool(value) {
        this._isPool = value;
    }

    get connectionSetting() {
        return this._connectionSetting;
    }

    set connectionSetting(value) {
        this._connectionSetting = value;
    }

    get sqlInterceptor() {
        return this._sqlInterceptor;
    }

    set sqlInterceptor(value) {
        this._sqlInterceptor = value;
    }

    get connectionStatus() {
        return this._connectionStatus;
    }

    set connectionStatus(value) {
        this._connectionStatus = value;
    }

    get hasTransaction() {
        return this._hasTransaction;
    }

    set hasTransaction(value) {
        this._hasTransaction = value;
    }

    get errorStack() {
        return this._errorStack;
    }

    set errorStack(value) {
        this._errorStack = value;
    }

    get connection() {
        return this._connection;
    }

    set connection(value) {
        this._connection = value;
    }

    get useConnectionPooling() {
        return this._useConnectionPooling;
    }

    set useConnectionPooling(value) {
        this._useConnectionPooling = value;
        BaseLib.setPoolName(this.poolName);
    }

    get poolName() {
        return "_dataBasePool_" + (this.useConnectionPooling ? this.useConnectionPooling : "NoPool");
    }

    static setPoolName(poolName) {
        if (BaseLib.poolNameList.indexOf(poolName) === -1) {
            BaseLib.poolNameList.push(poolName);
        }
    }

    static get poolNameList() {
        if (BaseLib._poolNameList == null) {
            BaseLib._poolNameList = new Array();
        }

        return BaseLib._poolNameList;
    }

    getConnectionFunction() { }

    createPool() { }

    createPoolConnection() {
        var _this = this;
        this.errorStack = new Error();
        return new Promise(function (resolver, reject) {
            _this.createPool();
            _this.getConnectionFunction()(function (err, connection) {
                _this.connection = connection;
                _this._addError();
                _this._connectionConnect(resolver, reject, err);
            });
        });
    }

    createSingleConnection() {
        var _this = this;
        this.errorStack = new Error();
        this.connection = this.getConnectionFunction()(this.connectionSetting);
        return new Promise(function (resolver, reject) {
            _this._addError();
            _this.connection.connect(function (err) {
                _this._connectionConnect(resolver, reject, err);
            });
        });
    }

    getTableStruct(tablename) { }

    beginTransaction() { }

    commitTransaction() { }

    rollbackTransaction() { }

    close(callback, poolConnectionIsDestroy) {
        if (this.connection == null) {
            return;
        }

        this.connectionStatus = this.connectionStatusEnum.Close;
        BaseLib.removeConnectionRecord(this.poolName);
        BaseLib.removeConnectionObject(this.connection);
    }

    create() {
        if (this.connection != null) {
            return new Promise(function (resolver, reject) {
                global.setTimeout(resolver);
            });
        } else {
            return this.isPool ? this.createPoolConnection() : this.createSingleConnection();
        }
    }

    query(sql, params) { }

    _connectionEnd(err) {
        if (err) {
            DebugDBHelper("连接结束出错：" + err);
        }
    }

    _handleDataBaseConnectionError(qerr, resolver, reject, sql, params) {
        if (qerr) {
            let errmsg = null;
            this.close(null, true);
            switch (qerr.code) {
                case "PROTOCOL_CONNECTION_LOST":
                case "PROTOCOL_ENQUEUE_AFTER_FATAL_ERROR":
                case "PROTOCOL_ENQUEUE_AFTER_QUIT":
                    let _this = this;
                    global.setTimeout(function () {
                        _this.query(sql, params).then(function (tempValue) {
                            resolver(tempValue);
                        },
                            function (tempErr) {
                                reject(tempErr);
                            });
                    });
                    break;
                case "ECONNREFUSED":
                    errmsg = "数据库连接失败，请联系管理员";
                    global.setTimeout(function () {
                        reject(errmsg);
                    });
                    break;
                default:
                    global.setTimeout(function () {
                        reject(qerr);
                    });
                    break;
            }
        }
    }

    limit(start, count) { }

    toNumber(value) { }

    IF(condition, truedata, falsedata) { }

    IFNULL(expression, altValue) { }

    SUBSTRING_INDEX(field, split, index) { }

    Like(value) {
        if (StringUtils.isNullOrEmpty(value)) return "";
        var str = this.sqlEscape(value, null, null, false);
        var buffer = new StringBuilder();
        var arr = str.split('');
        for (var i = 0, count = arr.length; i < count; i++) {
            var ch = arr[i];
            if (ch == '%' || ch == '\\' || ch == '_') {
                buffer.append('\\').append(ch);
            } else {
                buffer.append(ch);
            }
        }
        return buffer.toString();
    }

    sqlEscape(value, stringifyObjects, timeZone, isAddMake) {
        isAddMake = isAddMake == null ? true : isAddMake;
        let result = SqlString.escape(value, stringifyObjects, timeZone);
        if (isAddMake)
            return result;
        else {
            if (result.startsWith('\'') && result.endsWith('\'')) {
                return result.substr(1, result.length - 2);
            }

            return result;
        }
    }

    _addError() {
        if (this.connection && !this.connection.__addErrorListener) {
            let fun = this.isPool ? this._onPoolConnectError.bind(this) : this._onSingleConnectError.bind(this);
            this.connection.on('error', fun);
            this.connection.__addErrorListener = true;
        }
    }

    _onPoolConnectError(error) {
        DebugDBHelper("数据库连接池连接错误：" + error);
    }

    _onSingleConnectError(error) {
        DebugDBHelper("单个数据库连接错误：" + error);
    }

    _connectionConnect(resolver, reject, err) {
        if (this.connection) {
            this.connection._stack = this.errorStack.stack;
            this.connection._startDate = new Date();
            this.connection._cuid = Cuid.newCuidString();
            this.connection._isPool = this.isPool;

            BaseLib.addConnectionRecord(this.poolName);
            BaseLib.addConnectionObject(this.connection);
        }

        if (err) {
            this.close(null, true);
        } else {
            this.connectionStatus = this.connectionStatusEnum.Open;
        }

        if (err == null)
            resolver();
        else
            reject(err);
    }

    _queryFormat(query, values) {
        if (!values) {
            if (this.sqlInterceptor) this.sqlInterceptor(this, { sql: query, values: null });
            return query;
        }

        let _this = this;
        let encodeSql = query.replace(/\:(\w+)/g, function (txt, key) {
            if (values.hasOwnProperty(key)) return _this.sqlEscape(values[key], false);
            else return txt;
        }.bind(this));

        if (this.sqlInterceptor) this.sqlInterceptor(this, { sql: encodeSql, values: values });
        return encodeSql;
    }

    static addConnectionRecord(poolName) {
        let value = BaseLib.getConnectionRecord(poolName);
        value += 1;
        BaseLib.getPoolConnectionRecordMap().set(poolName, value);
    }

    static removeConnectionRecord(poolName) {
        let value = BaseLib.getConnectionRecord(poolName);
        value -= 1;
        BaseLib.getPoolConnectionRecordMap().set(poolName, value);
    }

    static getPoolInfoList() {
        let poolNameList = BaseLib.poolNameList;
        let poolInfoList = new Array();
        for (let i = 0; i < poolNameList.length; i++) {
            let poolName = poolNameList[i];
            let nowConnectionCount = BaseLib.getConnectionRecord(poolName);
            let poolInfo = {
                name: poolName,
                nowConnectionCount: nowConnectionCount
            };
            poolInfoList.push(poolInfo);
        }
        return poolInfoList;
    }

    static getPoolConnectionRecordMap() {
        if (BaseLib._poolConnectionRecordMap == null)
            BaseLib._poolConnectionRecordMap = new Map();
        return BaseLib._poolConnectionRecordMap;
    }

    static getConnectionRecord(poolName) {
        let value = BaseLib.getPoolConnectionRecordMap().get(poolName);
        if (value == null)
            value = 0;
        return value;
    }

    static addConnectionObject(connection) {
        if (!connection) {
            return;
        }

        let map = BaseLib.getConnectionObjectMap();
        if (map.has(connection._cuid)) {
            Log.writeDebug("连接ID重复：" + connection._cuid + " 堆栈：" + connection._stack);
        } else {
            map.set(connection._cuid, connection);
        }
    }

    static removeConnectionObject(connection) {
        if (!connection) {
            return;
        }

        let map = BaseLib.getConnectionObjectMap();
        if (map.has(connection._cuid)) {
            map.delete(connection._cuid);
        } else {
            Log.writeDebug("连接ID不存在：" + connection._cuid + " 堆栈：" + connection._stack);
        }

        BaseLib._printfTimeoutConnectionObject();
    }

    static getConnectionObjectMap() {
        if (BaseLib._connectionObjectMap == null)
            BaseLib._connectionObjectMap = new Map();
        return BaseLib._connectionObjectMap;
    }

    static _printfTimeoutConnectionObject() {
        let map = BaseLib.getConnectionObjectMap();
        let nowDateTicks = new Date().getTime();
        let dbConnectOccupyTimeout = ApracConfig.DBConnectOccupyTimeout == null ? 600000 : ApracConfig.DBConnectOccupyTimeout;
        map.forEach(function (value, key, map) {
            if (value._printfTimeoutConnectionObject == null && value._isPool && value._startDate.getTime() + dbConnectOccupyTimeout < nowDateTicks) {
                Log.writeDebug("连接池连接ID超时：" + value._cuid + " 堆栈：" + value._stack);
                value._printfTimeoutConnectionObject = true;
            }
        });
    }
}

module.exports = BaseLib;

==> apracjs-17.1.17.tgz: package/lib/database/base/baseLib.js <==
﻿var StringBuilder = require('../../common/stringBuilder');
var Url = require('url');
var DebugDBHelper = require('debug')('aprac:DBHelper');
var SqlString = require('sqlstring');
var StringUtils = require("../../common/stringUtils");
var Cuid = require('../../common/cuid');
var Log = require('../../log/log4Helper').getInstance();
var ApracConfig = require('../../config').apracConfig;

class BaseLib {
    constructor(connectionSetting) {
        this.connectionSetting = connectionSetting;
        this.connectionStatusEnum = new Object();
        this.connectionStatusEnum.Open = 1;
        this.connectionStatusEnum.Close = 0;
        this._initConnectionSetting();
        //BaseLib._startKillLockConnectionService();
    }

    _initConnectionSetting() {
        let uriInfo = Url.parse(this.connectionSetting, true);
        if (uriInfo.query.useConnectionPooling) {
            this.isPool = true;
            this.useConnectionPooling = uriInfo.query.useConnectionPooling;
        } else {
            this.isPool = false;
        }

        this.hostname = uriInfo.hostname;
        this.port = uriInfo.port;
        this.user = uriInfo.auth.split(':')[0];
        this.password = uriInfo.auth.split(':')[1];
        this.database = uriInfo.pathname.indexOf('/') == 0 ? uriInfo.pathname.substr(1) : uriInfo.pathname;
        this.connectionlimit = uriInfo.query && uriInfo.query.connectionLimit ? parseInt(uriInfo.query.connectionLimit) : 10;
        if (isNaN(this.connectionlimit)) {
            this.connectionlimit = 10;
        }
        this.connection = null;
        this.sqlInterceptor = null;
        this.connectionStatus = this.connectionStatusEnum.Close; //0：未连接 1：已连接
        this.hasTransaction = false;
    }

    get connectionlimit() {
        return this._connectionlimit;
    }

    set connectionlimit(value) {
        this._connectionlimit = value;
    }

    get hostname() {
        return this._hostname;
    }

    set hostname(value) {
        this._hostname = value;
    }

    get port() {
        return this._port;
    }

    set port(value) {
        this._port = value;
    }

    get user() {
        return this._user;
    }

    set user(value) {
        this._user = value;
    }

    get password() {
        return this._password;
    }

    set password(value) {
        this._password = value;
    }

    get database() {
        return this._database;
    }

    set database(value) {
        this._database = value;
    }

    get isPool() {
        return this._isPool;
    }

    set isPool(value) {
        this._isPool = value;
    }

    get connectionSetting() {
        return this._connectionSetting;
    }

    set connectionSetting(value) {
        this._connectionSetting = value;
    }

    get sqlInterceptor() {
        return this._sqlInterceptor;
    }

    set sqlInterceptor(value) {
        this._sqlInterceptor = value;
    }

    get connectionStatus() {
        return this._connectionStatus;
    }

    set connectionStatus(value) {
        this._connectionStatus = value;
    }

    get hasTransaction() {
        return this._hasTransaction;
    }

    set hasTransaction(value) {
        this._hasTransaction = value;
    }

    get errorStack() {
        return this._errorStack;
    }

    set errorStack(value) {
        this._errorStack = value;
    }

    get connection() {
        return this._connection;
    }

    set connection(value) {
        this._connection = value;
    }

    get useConnectionPooling() {
        return this._useConnectionPooling;
    }

    set useConnectionPooling(value) {
        this._useConnectionPooling = value;
        BaseLib.setPoolName(this.poolName);
    }

    get poolName() {
        return "_dataBasePool_" + (this.useConnectionPooling ? this.useConnectionPooling : "NoPool");
    }

    static setPoolName(poolName) {
        if (BaseLib.poolNameList.indexOf(poolName) === -1) {
            BaseLib.poolNameList.push(poolName);
        }
    }

    static get poolNameList() {
        if (BaseLib._poolNameList == null) {
            BaseLib._poolNameList = new Array();
        }

        return BaseLib._poolNameList;
    }

    getConnectionFunction() { }

    createPool() { }

    createPoolConnection() {
        var _this = this;
        this.errorStack = new Error();
        return new Promise(function(resolver, reject) {
            _this.createPool();
            _this.getConnectionFunction()(function(err, connection) {
                _this.connection = connection;
                _this._addError();
                _this._connectionConnect(resolver, reject, err);
            });
        });
    }

    createSingleConnection() {
        var _this = this;
        this.errorStack = new Error();
        this.connection = this.getConnectionFunction()(this.connectionSetting);
        return new Promise(function(resolver, reject) {
            _this._addError();
            _this.connection.connect(function(err) {
                _this._connectionConnect(resolver, reject, err);
            });
        });
    }

    getTableStruct(tablename) { }

    beginTransaction() { }

    commitTransaction() { }

    rollbackTransaction() { }

    close(callback, poolConnectionIsDestroy) {
        if (this.connection == null) {
            return;
        }

        this.connectionStatus = this.connectionStatusEnum.Close;
        BaseLib.removeConnectionRecord(this.poolName);
        BaseLib.removeConnectionObject(this.connection);
    }

    create() {
        if (this.connection != null) {
            return new Promise(function(resolver, reject) {
                global.setTimeout(resolver);
            });
        } else {
            return this.isPool ? this.createPoolConnection() : this.createSingleConnection();
        }
    }

    query(sql, params) { }

    _connectionEnd(err) {
        if (err) {
            DebugDBHelper("连接结束出错：" + err);
        }
    }

    _handleDataBaseConnectionError(qerr, resolver, reject, sql, params) {
        if (qerr) {
            //附加日志，方便排查可能存在的泄漏问题
            let fullErrorMsg = " __rawSQL:【" + JSON.stringify(sql) + "】 __stack:【" + (this._errorStack ? this._errorStack.stack : "") + "】";

            if (typeof qerr === "string")
                qerr += fullErrorMsg;
            else if (typeof qerr === "object") {
                qerr.__fullErrorMsg = fullErrorMsg;

                if (qerr.message && typeof qerr.message === "string")
                    qerr.message += fullErrorMsg;

                if (qerr.stack && typeof qerr.stack === "string")
                    qerr.stack += fullErrorMsg;
            }

            //统一处理错误
            let errmsg = null;
            this.close(null, true);
            switch (qerr.code) {
                case "PROTOCOL_CONNECTION_LOST":
                case "PROTOCOL_ENQUEUE_AFTER_FATAL_ERROR":
                case "PROTOCOL_ENQUEUE_AFTER_QUIT":
                    let _this = this;
                    global.setTimeout(function() {
                        _this.query(sql, params).then(function(tempValue) {
                            resolver(tempValue);
                        },
                            function(tempErr) {
                                reject(tempErr);
                            });
                    });
                    break;
                case "ECONNREFUSED":
                    errmsg = "数据库连接失败，请联系管理员";
                    global.setTimeout(function() {
                        reject(errmsg);
                    });
                    break;
                default:
                    global.setTimeout(function() {
                        reject(qerr);
                    });
                    break;
            }
        }
    }

    limit(start, count) { }

    toNumber(value) { }

    IF(condition, truedata, falsedata) { }

    IFNULL(expression, altValue) { }

    SUBSTRING_INDEX(field, split, index) { }

    Like(value) {
        if (StringUtils.isNullOrEmpty(value)) return "";
        var str = this.sqlEscape(value, null, null, false);
        var buffer = new StringBuilder();
        var arr = str.split('');
        for (var i = 0, count = arr.length; i < count; i++) {
            var ch = arr[i];
            if (ch == '%' || ch == '\\' || ch == '_') {
                buffer.append('\\').append(ch);
            } else {
                buffer.append(ch);
            }
        }
        return buffer.toString();
    }

    sqlEscape(value, stringifyObjects, timeZone, isAddMake) {
        isAddMake = isAddMake == null ? true : isAddMake;
        let result = SqlString.escape(value, stringifyObjects, timeZone);
        if (isAddMake)
            return result;
        else {
            if (result.startsWith('\'') && result.endsWith('\'')) {
                return result.substr(1, result.length - 2);
            }

            return result;
        }
    }

    _addError() {
        if (this.connection && !this.connection.__addErrorListener) {
            let fun = this.isPool ? this._onPoolConnectError.bind(this) : this._onSingleConnectError.bind(this);
            this.connection.on('error', fun);
            this.connection.__addErrorListener = true;
        }
    }

    _onPoolConnectError(error) {
        DebugDBHelper("数据库连接池连接错误：" + error);
    }

    _onSingleConnectError(error) {
        DebugDBHelper("单个数据库连接错误：" + error);
    }

    _connectionConnect(resolver, reject, err) {
        if (this.connection) {
            this.connection._stack = this.errorStack.stack;
            this.connection._startDate = new Date();
            this.connection._cuid = Cuid.newCuidString();
            this.connection._isPool = this.isPool;

            BaseLib.addConnectionRecord(this.poolName);
            BaseLib.addConnectionObject(this.connection);
        }

        if (err) {
            this.close(null, true);
        } else {
            this.connectionStatus = this.connectionStatusEnum.Open;
        }

        if (err == null)
            resolver();
        else
            reject(err);
    }

    _queryFormat(query, values) {
        if (!values) {
            if (this.sqlInterceptor) {
                this.sqlInterceptor(this, { sql: query, values: null });
            }
            return query;
        }

        let _this = this;
        let encodeSql = query.replace(/\:(\w+)/g, function(txt, key) {
            if (values.hasOwnProperty(key)) {
                return _this.sqlEscape(values[key], false);
            } else {
                return txt;
            }
        }.bind(this));

        if (this.sqlInterceptor) {
            this.sqlInterceptor(this, { sql: encodeSql, values: values });
        }
        return encodeSql;
    }

    static addConnectionRecord(poolName) {
        let value = BaseLib.getConnectionRecord(poolName);
        value += 1;
        BaseLib.getPoolConnectionRecordMap().set(poolName, value);
    }

    static removeConnectionRecord(poolName) {
        let value = BaseLib.getConnectionRecord(poolName);
        value -= 1;
        BaseLib.getPoolConnectionRecordMap().set(poolName, value);
    }

    static getPoolInfoList() {
        let poolNameList = BaseLib.poolNameList;
        let poolInfoList = new Array();
        for (let i = 0; i < poolNameList.length; i++) {
            let poolName = poolNameList[i];
            let nowConnectionCount = BaseLib.getConnectionRecord(poolName);
            let poolInfo = {
                name: poolName,
                nowConnectionCount: nowConnectionCount
            };
            poolInfoList.push(poolInfo);
        }
        return poolInfoList;
    }

    static getPoolConnectionRecordMap() {
        if (BaseLib._poolConnectionRecordMap == null)
            BaseLib._poolConnectionRecordMap = new Map();
        return BaseLib._poolConnectionRecordMap;
    }

    static getConnectionRecord(poolName) {
        let value = BaseLib.getPoolConnectionRecordMap().get(poolName);
        if (value == null)
            value = 0;
        return value;
    }

    static addConnectionObject(connection) {
        if (!connection) {
            return;
        }

        let map = BaseLib.getConnectionObjectMap();
        if (map.has(connection._cuid)) {
            Log.writeDebug("连接ID重复：" + connection._cuid + " 堆栈：" + connection._stack);
        } else {
            map.set(connection._cuid, connection);
        }
    }

    static removeConnectionObject(connection) {
        if (!connection) {
            return;
        }

        let map = BaseLib.getConnectionObjectMap();
        if (map.has(connection._cuid)) {
            map.delete(connection._cuid);
        } else {
            Log.writeDebug("连接ID不存在：" + connection._cuid + " 堆栈：" + connection._stack);
        }

        BaseLib._printfTimeoutConnectionObject();
    }

    static getConnectionObjectMap() {
        if (BaseLib._connectionObjectMap == null)
            BaseLib._connectionObjectMap = new Map();
        return BaseLib._connectionObjectMap;
    }

    static _printfTimeoutConnectionObject() {
        let map = BaseLib.getConnectionObjectMap();
        let nowDateTicks = new Date().getTime();
        let dbConnectOccupyTimeout = ApracConfig.DBConnectOccupyTimeout == null ? 600000 : ApracConfig.DBConnectOccupyTimeout;
        map.forEach(function(value, key, map) {
            if (value._printfTimeoutConnectionObject == null && value._isPool && value._startDate.getTime() + dbConnectOccupyTimeout < nowDateTicks) {
                Log.writeDebug("连接池连接ID超时：" + value._cuid + " 堆栈：" + value._stack);
                value._printfTimeoutConnectionObject = true;
            }
        });
    }

    static _startKillLockConnectionService() {
        if (BaseLib._startKillLockConnection) return;

        BaseLib._startKillLockConnection = true;
        BaseLib._killLockConnectionService();
    }

    static _killLockConnectionService() {
        //连接池死锁检测需求保留
    }
}

module.exports = BaseLib;
